//A component of FEAST.
//Copyright (c) 2003 David Elworthy.
//For information, visit www.friendlymoose.com
package com.friendlymoose.feast;
import java.awt.Color;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

/** Table model holding thresholds and colours. */
public class ColourTable extends AbstractTableModel
{
	/** Number of rows */
	private int rows;
	
	/** Threshold on each row. */
	private Integer levels[];
	
	/** Colour for each row. */
	private Color colours[];
	
	/** Title of the columns */
	private String columnName[] = { "Level", "Colour" };

	/** Supporting class: a table cell represented as a label in the right colour. */
	public static class ColourRenderer extends JLabel implements TableCellRenderer
	{
		/** Make a color renderer */ 
		public ColourRenderer()
		{
			super();
			setOpaque(true); //MUST do this for background to show up.
		}

		/** Get a table cell renderer with background set to the colour
		 * passed as value.
		 * @param table (table)
		 * @param colour Colour to set.
		 * @param isSelected (isSelected)
		 * @param hasFocus (hasFocus)
		 * @param row (row)
		 * @param column (column)
		 * @return renderer component
		 */
		public Component getTableCellRendererComponent
		(JTable table, Object colour, boolean isSelected, boolean hasFocus,
		 int row, int column)
		 {
			setBackground((Color)colour);
			return this;
		}
	}

	/** Supporting class used to edit coloured cells. */
	public static class ColourEditor extends DefaultCellEditor
	{
		/** Current colour of the cell. */
		Color currentColour = null;

		/** Make the colour editor.
		 * 
		 * @param b Button for the editor
		 */
		public ColourEditor(JButton b)
		{
			super(new JCheckBox()); //Unfortunately, the constructor
									//expects a check box, combo box,
									//or text field.
			editorComponent = b;
			setClickCountToStart(1); //This is usually 1 or 2.

			//Must do this so that editing stops when appropriate.
			b.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					fireEditingStopped();
				}
			});
		}
	
		/** Standard method. */
		protected void fireEditingStopped()
		{
			super.fireEditingStopped();
		}

		/** Get the cell value.
		 *  @return the colour for the cell.
		 */
		public Object getCellEditorValue()
		{
			return currentColour;
		}

		/** Get a table cell renderer.
		 * @param table (table)
		 * @param value (value)
		 * @param isSelected (isSelected)
		 * @param row (row)
		 * @param column (column)
		 * @return renderer component
		 */
		public Component getTableCellEditorComponent
		(JTable table, Object value, boolean isSelected, int row, int column)
		{
			((JButton)editorComponent).setText(value.toString());
			currentColour = (Color) value;
			return editorComponent;
		}
	}

	/** A colour table. Provides the model for a table mapping levels to colours.
	 * 
	 * @param rows Number of rows
	 */
	public ColourTable(int rows)
	{
		this.rows = rows;
		levels  = new Integer[rows];
		colours = new Color[rows];
		for (int i = 0; i < rows; ++i)
		{
			levels[i]  = new Integer(0);
			colours[i] = Color.WHITE;
		}
	}
	
	/** Set the levels and colours
	 * @param l Array of levels
	 * @param c Array of colours
	 */
	public void set(Integer l[], Color c[])
	{
		int max = l.length;
		if (max > levels.length)
		{
			max = levels.length;
		}
		
		int i;
		for (i = 0; i < max; ++i)
		{
			levels[i]  = l[i];
			colours[i] = c[i];
		}
		for (; i < levels.length; ++i)
		{
			levels[i]  = new Integer(0);
			colours[i] = Color.WHITE;
		}
	}

	/** Standard method.
	 *  @param row Row to test.
	 *  @param col Column to test.
	 *  @return true if cell may be edited.
	 */	
	public boolean isCellEditable(int row, int col)
	{
		return true;
	}

	/** Get name for a column
	 * @param col Column number.
	 * @return Name of the column.
	 */
	public String getColumnName(int col)
	{
		return columnName[col];
	}

	/** Set a value.
	 * @param value Value to set.
	 * @param row Row in which to set it.
	 * @param col Column in which to set it.
	 */
	public void setValueAt(Object value, int row, int col)
	{
		if (value == null)
		{
			levels[row] = null;
			colours[row] = Color.WHITE;
		}
		else if (col == 0)
		{
			levels[row] = (Integer) value;
			colours[row] = Color.WHITE;
		}
		else if (col == 1)
		{
			colours[row] = (Color) value;
		}
		fireTableCellUpdated(row, col);
	}
	
	/** Get the levels.
	 * 
	 * @return array of levels.
	 */
	public Integer[] getLevels()
	{
		return levels;
	}
	
	/** Get the colours.
	 * 
	 * @return array of colours.
	 */
	public Color[] getColours()
	{
		return colours;
	}

	/** Get number of rows.
	 * @return number of rows.
	 */
    public int getRowCount()
    {
        return rows;
    }

	/** Get number of columns.
	 * @return number of columns.
	 */
    public int getColumnCount()
    {
        return 2;
    }

	/** Get value in a cell.
	 *  @param row Row number.
	 *  @param col Column number.
	 *  @return Value in the cell - either an Integer or a Color.
	 */
    public Object getValueAt(int row, int col)
    {
    	if (col == 0)
    	{
    		return levels[row];
    	}
    	else if (levels[row] != null)
		{
			return colours[row];
		}
    	else
    	{
    		return null;
    	}
    }

	/** Get class of a column.
	 *  @param col Column number.
	 *  @return Class for the column - either Integer or Color.
	 */
	public Class getColumnClass(int col)
	{
		return (col == 0) ? Integer.class : Color.class;
	}

	/** Set up a renderer.
	 *  
	 * @param table Table for the renderer.
	 */
	public static void setUpColorRenderer(JTable table)
	{
		table.setDefaultRenderer(Color.class, new ColourRenderer());
	}


	/** Set up a colour editor.
	 * 
	 * @param table Table wanting the colour editor
	 */
	public static void setUpColourEditor(JTable table)
	{
		// First, set up the button that brings up the dialog.
	  	final JButton button = new JButton("")
	  	{
	  		public void setText(String s)
	  		{
	  			// Button never shows text -- only color.
		  	}
	  	};
	  	button.setBackground(Color.white);
	  	button.setBorderPainted(false);
	  	button.setMargin(new Insets(0,0,0,0));

	  	// Now create an editor to encapsulate the button, and
	  	// set it up as the editor for all Color cells.
	  	final ColourEditor colourEditor = new ColourEditor(button);
	  	table.setDefaultEditor(Color.class, colourEditor);

	  	// Set up the dialog that the button brings up.
	  	final JColorChooser colorChooser = new JColorChooser();
	  	ActionListener okListener = new ActionListener()
	  	{
	  		public void actionPerformed(ActionEvent e)
	  		{
				colourEditor.currentColour = colorChooser.getColor();
		  	}
	  	};
	  	final JDialog dialog
	  		= JColorChooser.createDialog
	  		  (button, "Pick a Color", true, colorChooser, okListener, null);

	  	// Here's the code that brings up the dialog.
	  	button.addActionListener(new ActionListener()
	  	{
			public void actionPerformed(ActionEvent e)
			{
			 	button.setBackground(colourEditor.currentColour);
			  	colorChooser.setColor(colourEditor.currentColour);
			  	// Without the following line, the dialog comes up
			  	// in the middle of the screen.
			  	dialog.setLocationRelativeTo(button);
			  	dialog.show();
		  	}
	  	});
  	}
}
