//A component of FEAST.
//Copyright (c) 2003 David Elworthy.
//For information, visit www.friendlymoose.com
package com.friendlymoose.feast;
import java.awt.Color;
import java.awt.Component;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

/** Renderer for table cells which displays then in a given colour. */
public class ColouredRenderer extends DefaultTableCellRenderer
implements PropertyBearing
{
	/** Mapping from threshold to colour. */
	private TreeMap mapping = new TreeMap();
	
	/** Create renderer. */
	public ColouredRenderer()
	{
		super();
	}
	
	/** Assign colours for the renderer.
	 *  
	 * @param levels Array of integer thresholds.
	 * @param colours Colours to use at each threshold.
	 */
	public void assignColours(Integer levels[], Color colours[])
	{
		mapping.clear();
		for (int i = 0; i < levels.length; ++i)
		{
			int val = (levels[i] == null) ? 0 : levels[i].intValue();
			if (val > 0 && colours[i] != null)
			{
				mapping.put(levels[i], colours[i]);
			}
		}
	}
    
    /** Assign the colour for a level.
     * @param level The threshold for the level.
     * @param colour The colour to display when the threshold is reached.
     */
	public void assignColour(int level, Color colour)
	{
		mapping.put(new Integer(level), colour);
	}
	
	/** Get the values of the levels.
	 * @return Array of integers for the level values
	 */
	public Integer[] getLevels()
	{
		return (Integer []) (mapping.keySet().toArray(new Integer[0]));
	}

	/** Get the array of colours.
	 * 
	 * @return Array of colours, paralleling the levels.
	 */
	public Color[] getColours()
	{
		return (Color []) (mapping.values().toArray(new Color[0]));
	}

	/** Get a table cell renderer.
	 * @param table (table)
	 * @param value (value)
	 * @param isSelected (isSelected)
	 * @param hasFocus (hasFocus)
	 * @param row (row)
	 * @param column (column)
	 * @return renderer component
	 */
	public Component getTableCellRendererComponent
	(JTable table, Object value, boolean isSelected, boolean hasFocus,
	 int row, int column)
	{
		if (column == 2)
		{
			try
			{
				int count = Integer.parseInt(table.getModel().getValueAt(row, column).toString());
				
				Color colour = Color.WHITE;
				for (Iterator i = mapping.entrySet().iterator(); i.hasNext(); )
				{
					Map.Entry entry = (Map.Entry) i.next();
					if (((Integer) entry.getKey()).intValue() > count)
					{
						break;
					}
					else
					{
						colour = (Color) entry.getValue();
					}
				}
	
				setBackground(colour);
			}
			catch (Exception e)
			{
				setBackground(Color.WHITE);
			}
	  }
    
	  setText((value == null) ? "" : value.toString());        
	  return this;
	}

    /** Load the properties.
     *  @param props Property holder.
     */
    public void loadProperties(Properties props)
    {
		int max = Integer.parseInt(props.getProperty("Colour#", "0"));
		mapping.clear();
		for (int i = 0; i < max; ++i)
		{
			String s = props.getProperty("Colour_L"+i, null);
			String v = props.getProperty("Colour_V"+i, null);
			if (s != null && v != null)
			{
				assignColour(Integer.parseInt(s), new Color(Integer.parseInt(v)));
			}
		}        
    }

    /** Save the properties.
     * @param props Property holder.
     */
    public void saveProperties(Properties props)
    {
		int max = mapping.size();
		props.setProperty("Colour#", Integer.toString(max));
    	
    	int c = 0;
		for (Iterator i = mapping.entrySet().iterator(); i.hasNext(); ++c)
		{
			Map.Entry entry = (Map.Entry) i.next();
			props.setProperty("Colour_L"+c, entry.getKey().toString());
			props.setProperty("Colour_V"+c,
					 Integer.toString(((Color) entry.getValue()).getRGB()));
		}
    }
}
