/*
 * <copyright>
 * Copyright  2000-2004, David Elworthy.
 * </copyright>
 *
 * $Header: $
 * Title:        Crypter
 * Created:      Feb 19, 2005
 * Description:  Encryption/decryption class.
 */

package com.friendlymoose.feast;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.KeySpec;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEParameterSpec;

/** A class for encryption and decryption of streams.
 *  PBE (password-based encryption) could be tightened up by using a different salt and iterations
 *  value.
 */
public class Crypter
{
    /** The Cipher for encoding. */
    private Cipher ecipher;
    
    /** The Cipher for dencoding. */
    private Cipher dcipher;
    
    /** Algorithm parameters needed for the cipher. */
    private AlgorithmParameterSpec paramSpec;
    
    /** The key */
    private SecretKey theKey;
    
    /** Salt used for PBE. */
    private byte salt[] = { (byte)0x30, (byte)0x29, (byte)0x11, (byte)0x22,
                            (byte)0x14, (byte)0x11, (byte)0x25, (byte)0x01 };
    
    /** Iterations used for PBE. */
    private int pbeIters = 8;
    
    /** Create a crypter.
     *  @param pAlgName Algorithm name. Currently supports DES, DESede and PBE.
     *  @param pKey Crypto key. For DES, this must be at least 8 bytes. For DESede, it must be at
     *  least 24 bytes. For PBE, it is a pass phrase.
     *  @throws Exception for bad algorithm or parameters
     */
    public Crypter(String pAlgName, String pKey) throws Exception
    {
        KeySpec keySpec;
        String algorithm;
        paramSpec = null;
        
        if (pAlgName.equals("DES"))
        {
            keySpec = new javax.crypto.spec.DESKeySpec(pKey.getBytes());
            algorithm = "DES";
        }
        else if (pAlgName.equals("DESede"))
        {
            keySpec = new javax.crypto.spec.DESedeKeySpec(pKey.getBytes());
            algorithm = "DESede";
        }
        else if (pAlgName.equals("PBE"))
        {
            keySpec = new javax.crypto.spec.PBEKeySpec(pKey.toCharArray(), salt, pbeIters);
            algorithm = "PBEWithMD5AndDES";
            paramSpec = new PBEParameterSpec(salt, pbeIters);
        }
        else
        {
            throw new Exception("Unknown crypto type");
        }
            
        // Create the key representation and cipher
        theKey = SecretKeyFactory.getInstance(algorithm).generateSecret(keySpec);
        ecipher = Cipher.getInstance(theKey.getAlgorithm());
        dcipher = Cipher.getInstance(theKey.getAlgorithm());
        ecipher.init(Cipher.ENCRYPT_MODE, theKey, paramSpec);
        dcipher.init(Cipher.DECRYPT_MODE, theKey, paramSpec);
    }
    
    /** Validate.
     *  This checks the key was correct by taking an encoded string and what it should decode to.
     *  @param pEncoded Encoded string.
     *  @param pDecoded What it should decode to.
     *  @return True if the string decodes correctly.
     *  @throws Exception on decoding error.
     */
    public boolean validate(String pEncoded, String pDecoded) throws Exception
    {
        return crypt(pEncoded, true).equals(pDecoded);
    }
    
    /** Encrypt or decrypt a string.
     *  @param pSource Source string.
     *  @param pDecrypt True to decrypt, false to encrypt.
     *  @return The resulting string.
     *  @throws Exception on decoding error.  
     */
    public String crypt(String pSource, boolean pDecrypt) throws Exception
    {
        InputStream in = new ByteArrayInputStream(pSource.getBytes());
        OutputStream out = new ByteArrayOutputStream();
        decode(in, out, pDecrypt);
        return out.toString();
    }
    
    /** Decode a stream to a stream.
     *  @param pIn Stream to decode from.
     *  @param pOut Stream to decode into.
     *  @param pDecrypt True to decrypt, false to encrypt.
     *  @throws Exception on decoding error.
     */
    public void decode(InputStream pIn, OutputStream pOut, boolean pDecrypt) throws Exception
    {
        CipherInputStream in = new CipherInputStream(pIn, (pDecrypt) ? dcipher : ecipher);
        
        int i;
        while ((i = in.read()) != -1)
        {
            pOut.write(i);
        }
    }
    
    /** Encrypt a file.
     *  @param pFileName File name.
     *  @param pExt Extension for encrypted files.
     *  @param pMangle True to mangle names.
     *  @throws Exception on error.
     */
    public void encryptFile(String pFileName, String pExt, boolean pMangle)
    throws Exception
    {
        if (!pFileName.endsWith(pExt))
        {
            String target = (pMangle) ? crypt(pFileName, false)+pExt : pFileName+pExt;

            InputStream  in  = new BufferedInputStream (new FileInputStream(pFileName));
            OutputStream out = new BufferedOutputStream(new FileOutputStream(target));
            decode(in, out, false);
            in.close();
            out.close();
                    
            // Delete the old file
            (new File(pFileName)).delete();
        }
    }

    
    /** Decrypt a file.
     *  @param pFileName File name.
     *  @param pExt Extension for encrypted files.
     *  @param pMangle True to mangle names.
     *  @throws Exception on error.
     */
    public void decryptFile(String pFileName, String pExt, boolean pMangle)
    throws Exception
    {
        if (pFileName.endsWith(pExt))
        {
            String target = pFileName.substring(0, pFileName.length() - pExt.length());

            if (pMangle)
            {
                target = crypt(target, true);
            }
            InputStream  in  = new BufferedInputStream (new FileInputStream(pFileName));
            OutputStream out = new BufferedOutputStream(new FileOutputStream(target));
            decode(in, out, true);
            in.close();
            out.close();
                
            // Delete the old file
            (new File(pFileName)).delete();
        }
    }
}
