//A component of FEAST.
//Copyright (c) 2003 David Elworthy.
//For information, visit www.friendlymoose.com
package com.friendlymoose.feast;
import java.io.File;
import java.util.LinkedList;

import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/** Model for the file explorer. */
public class ExplorerModel extends DefaultTreeModel 
implements TreeExpansionListener, TreeSelectionListener
{
	/** File system roots. There is a fake root above these. */
	private File[] roots;
    
    /** Where to list the files. */
    private Lister lister;

	/** Create an explorer model.
	 * 
	 * @param lister File lister displaying the current directory
	 */
    public ExplorerModel(Lister lister)
    {
    	// Create fake root
    	super(new ExplorerNode(null));
	
		this.lister = lister;
		
		// Add all filesystem roots
		roots = File.listRoots();
	    ExplorerNode root = (ExplorerNode) getRoot();
	    for (int i = 0; i < roots.length; i++)
        {
			ExplorerNode aroot = add(root, roots[i]);
			ExplorerNode fake = new ExplorerNode();
	    	add(aroot, fake); // allows expansion
	    }
    }

    /**
     * Get a list of roots.
     * @return One or more file system root directories.
     */
    public File[] getRoots()
    {
		return roots;
    }
    
    /** Add a child to a node.
     * @param parent The parent node.
     * @param child The child node.
     */
    public void add(ExplorerNode parent, ExplorerNode child)
    {
		insertNodeInto(child, parent, getChildCount(parent));
    }

	/** Add a file to a node.
	 * 
	 * @param parent Parent node.
	 * @param file File to add to the node.
	 * @return New node for this file.
	 */
    public ExplorerNode add(ExplorerNode parent, File file)
    {
		ExplorerNode us = new ExplorerNode(file);
		add(parent, us);
		return us;
    }

    /** Given a file/directory name, return a TreePath for it.
     * @param file The file/directory name
     * @return Tree path for the directory.
     */
    public TreePath getTreePath(String file)
    {
		File f = new File(file);
		return getTreePath(f);
    }

	/** Given a file/directory, return a TreePath for it.
     * @param file The file/directory object
     * @return Tree path for the directory.
	 */
    public TreePath getTreePath(File file)
    { 
		ExplorerNode n = getNodeForFile(file);
		Object[] fortreepath = getPathToRoot(n);
		return new TreePath(fortreepath);
   	}
    
    /** Given a File, return the node in this model that uses it, if any.
     * @param file A file to look for.
     * @return Node for the file, or null if not found.
     */
    public ExplorerNode getNodeForFile(File file)
    {
		LinkedList stack = new LinkedList();
		File f = file;
		do
        {
	    	String name = (f.getParent() == null) ? f.getPath() : f.getName();
            stack.add(name);
	    	f = f.getParentFile();
		} while (f != null);
	
		ExplorerNode node = (ExplorerNode) getRoot();
	
		while (stack.size() != 0)
		{
	    	String froot = (String) stack.removeLast();
	    
	    	if (node.getFile() != null)
	    	{
				populate(node, false);
	    	}
	    	
	    	for(int i = 0; i < node.getChildCount(); i++)
            {
				ExplorerNode n = (ExplorerNode)node.getChildAt(i);
				String name = n.toString();
				if (name.equals(froot))
                {
		    		node = n;
		    		break;
				}
	    	}
		}
		
		populate(node, false);
		return node;
    }

	/** Refresh the display for a given directory.
	 * 
	 * @param dir Directory to refresh.
	 */
	public void refresh(File dir)
	{
		populate(getNodeForFile(dir), true);
	}

	/** Refresh the display for a given directory node.
	 * 
	 * @param node Node to refresh.
	 */
	public void refresh(ExplorerNode node)
	{
		populate(node, true);
	}

	/** Refresh the display for everything
	 */
	public void refresh()
	{
		for (int i = 0 ; i < roots.length; ++i)
		{
			populate(getNodeForFile(roots[i]), true);
		}
	}

    /** Add all subdirectories to this node.
     * Ignore hidden status of directories.
     * @param parent The parent node.
     * @param repopulate If true the node will be populated, even if it was done previously (ie it does a refresh)
     */
    protected void populate(ExplorerNode parent, boolean repopulate)
    {
	
		if (repopulate || !parent.isPopulated())
        {
		    parent.removeAllChildren();
		    parent.setPopulated(false);
		} 
	
		if ((!repopulate && parent.isPopulated()) || parent.getFile() == null)
		{
			return;
		}
	
		// find children of this node
		if (parent.getFile().exists())
		{
			File children[] = parent.getFile().listFiles();
	
			// Add nodes to this parent if there are any
			if (children != null)
	        {
	            for (int i = 0; i < children.length; i++)
	            {
					File child = children[i];

					if (child.isDirectory())
	                {
						ExplorerNode childNode = add(parent, child);
						if (hasDirectoryChildren(child))
						{
							add(childNode, new ExplorerNode());
						}

					}
		    	}
			}
		}
		
		parent.setPopulated(true);
		nodeStructureChanged(parent);
    }

	/** Inspect each member of the directory, and if there are
	 * any further directories under it, report true
	 * @param dir Directory we are inspecting
	 * @return True if there are any children which are directories.
	 */
	private boolean hasDirectoryChildren(File dir)
	{
		if (dir == null)
		{
			return false;
		}
		File children[] = dir.listFiles();
		if (children != null)
		{
			for (int j = 0; j < children.length; j++)
			{
				if (children[j].isDirectory())
				{
					return true;
				}
			}
		}
		
		return false;
	}
	
    /** Get a node, but only if it has a non-null file.
     * This screens out the root.
     * @param path The path to search for.
     * @return the node with this path.
     */
    private ExplorerNode getNodeFromPath(TreePath path)
    {
		if (path != null)
		{
			ExplorerNode node = (ExplorerNode) path.getLastPathComponent();   
			if (node != null && node.getFile() != null)
			{
				return node;
			}
		}
		
		return null;
    }

	/** Handle tree expansion event by populating a node.
	 * @param event Expansion event
	 */
    public void treeExpanded(TreeExpansionEvent event)
    {
		ExplorerNode node = getNodeFromPath(event.getPath());
    	if (node != null)
    	{
			populate(node, false);
    	}
    }

	/** Handle tree collapse event.
	 * @param event Tree collapse event.
	 */
    public void treeCollapsed(TreeExpansionEvent event)
    {
    }

	/** Handle value change event on selection, by passing it to lister.
	 * @param event Selection event
	 */
    public void valueChanged(TreeSelectionEvent event)
    {
		ExplorerNode node = getNodeFromPath(event.getNewLeadSelectionPath());
		if (node != null)
        {
			// If we have a lister, reset it
            if (lister != null && !lister.isPinned())
            {
                lister.setList(node.getFile());
            }

            // Maybe populate node
			populate(node, false);
	    }
    }
    
    /** Report if a node if a leaf.
     *  @param o Node to test
     *  @return true for leaves.
     */
    public boolean isLeaf(Object o)
    {
    	return false;
    }
}
