//A component of FEAST.
//Copyright (c) 2003 David Elworthy.
//For information, visit www.friendlymoose.com
package com.friendlymoose.feast;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * FEAST - an image displayer and organizer
 */
public class Feast extends JFrame
implements MouseListener, PropertyBearing, ActionListener
{
	/** Glue class: handles selection in the list and displays an image.
	 */
	private static class ListerSelectionHandler
	implements ListSelectionListener
	{
		/** Image displayer */
		private ImagePanel target;
		
		/** File lister */
		private Lister     lister;
        
        /** Construct selection handler.
         * 
         * @param target The image displayer
         * @param lister The file lister
         */
		public ListerSelectionHandler(ImagePanel target, Lister lister)
		{
			this.target = target;
			this.lister = lister;
		}
        
        /** Value changed event handler
         * @param e The list selection event.
         */
		public void valueChanged(ListSelectionEvent e)
		{
			int index = lister.getSelectedIndex();
			String name = lister.getName(index);
			target.displayFile(name);
		}
	}
    
    /** Listener for the path set spinner. */
    private static class SpinnerListener implements ChangeListener
    {
        /** Back link to the spinner model. */
        private SpinnerNumberModel spinner;
        
        /** Back link to the path set. */
        private PathsTable paths;
        
        /** Constructor.
         *  @param pSpinner The spinner.
         *  @param pPaths Tha paths table.
         */
        public SpinnerListener(SpinnerNumberModel pSpinner, PathsTable pPaths)
        {
            spinner = pSpinner;
            paths = pPaths;
        }

        /** @param e The event.
         */
        public void stateChanged(ChangeEvent e)
        {
            paths.setPathSet(spinner.getNumber().intValue());
        }
    }
	
	/** Key codes for function keys */
	private static int codes[] =
	{
		KeyEvent.VK_F1, KeyEvent.VK_F2, KeyEvent.VK_F3, KeyEvent.VK_F4, KeyEvent.VK_F5,
		KeyEvent.VK_F6, KeyEvent.VK_F7, KeyEvent.VK_F8, KeyEvent.VK_F9, KeyEvent.VK_F10, 
		KeyEvent.VK_F11, KeyEvent.VK_F12
	};
	
	/** Key codes for digit equivalents */
	private static int digit[] =
	{
		KeyEvent.VK_1, KeyEvent.VK_2, KeyEvent.VK_3, KeyEvent.VK_4, KeyEvent.VK_5,
		KeyEvent.VK_6, KeyEvent.VK_7, KeyEvent.VK_8, KeyEvent.VK_9, KeyEvent.VK_0, 
		KeyEvent.VK_MINUS, KeyEvent.VK_EQUALS
	};
    
    /** Type of encryption. */
    private static String cryptType = "PBE";
	
	/** Refresher for the class.
	 */
	private Refresher refresher;
	
	/** Properties. */
	protected PropertyHolder props;
	
	/** File holding properties */
	protected File propLoc;
	
	/** UI objects. */
	private JSplitPane pane1, pane2;
	
	/** UI object */
	private JFileChooser chooser;
	
	/** File lister */
	private Lister lister;
    
    /** Image panel. */
    private ImagePanel imagePanel;
	
	/** Table of paths */
	private PathsTable paths;
	
	/** Delete handler */
	private DeleteAction deleter;
	
	/** Deleter menu item */
	private JMenuItem deleterMenu;
	
	/** Colour choosing dialogue */
	private ColourDialogue colours;
	
	/** UI objects */
	private JMenuItem pinMenu, sortNAMenu, sortNDMenu, sortSAMenu, sortSDMenu, sortSRMenu,
				fullPathsMenu, mangleMenu;
                
    /** The password manager. */
    private PasswordManager passwordManager;
    
    /** The crypter. */
    private Crypter crypter;
                
    /** Number of path sets */
    private int nSets = 10;
    
    /** Extension used on encrypted file names. */
    public static final String EncryptExt = ".feast";

    /** Tree adjuster. */
    private TreeAdjuster adjuster;
        
    /** Flag to indicate whether name mangling should be done on encryption. */
    // TODO: this is not yet implemented, as we need a way of making the names safe.
    // Probably do it by having an encrypted file which maps names to integers
    private boolean mangleNames = false;
	
	/** Top level FEAST class.
	 *  @param args Command line arguments. Provides the feast properties file name,
	 *
	 */
	public Feast(String args[])
	{
		try
		{
			javax.swing.UIManager.setLookAndFeel
				("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		}
		catch (Exception e)
		{}
		
		int nKeys = codes.length;
		
		// List the default extensions we will handle
		Extensions extensions = new Extensions();
		extensions.add(".jpg");
		extensions.add(".gif");
		extensions.add(".jpeg");
        extensions.add(EncryptExt);
		        
        // Make the main panel
        JPanel mainPanel = new JPanel();
		Container c = getContentPane();
		c.add(mainPanel);
		mainPanel.setLayout(new BorderLayout());
		
		chooser = new JFileChooser();

		// Make the image display panel. It is initially disabled.
		imagePanel = new ImagePanel(false);
		JScrollPane image = new JScrollPane(imagePanel);
		
		// Make a text field for displaying file count
		JLabel fileCounter = new JLabel(" ");
		
		// Make the file lister
		lister = new Lister(extensions, fileCounter);
		lister.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		lister.addListSelectionListener
							(new ListerSelectionHandler(imagePanel, lister));
		JScrollPane listPanel = new JScrollPane(lister);
		
		// Make directory tree displayer
		ExplorerModel explorerModel = new ExplorerModel(lister);
		ExplorerTree  explorer      = new ExplorerTree(explorerModel);
		if (explorerModel.getRoots().length == 1)
		{
			explorer.setRootVisible(false);
		}
		explorer.putClientProperty("JTree.lineStyle","Angled");
		explorer.addTreeExpansionListener(explorerModel);
		explorer.addTreeSelectionListener(explorerModel);
		JScrollPane treePanel = new JScrollPane(explorer);

		// Make a colouring renderer
		ColouredRenderer coloured = new ColouredRenderer();
		
		// Make the paths list
		paths = new PathsTable(nKeys, extensions, nSets);
		JTable pathTable = new JTable(paths);
		pathTable.addMouseListener(new MouseListener()
		{
			public void mouseClicked(MouseEvent e)
			{
				JTable table = (JTable) e.getComponent();
				PathsTable paths = (PathsTable) (table.getModel());
				int row = table.getSelectedRow();
				paths.setPath(row, -1, null);
			}
			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}
			public void mousePressed(MouseEvent e) {}
			public void mouseReleased(MouseEvent e) {}
		});
		
		// Set properties of the paths list
		TableColumnModel cmodel = pathTable.getColumnModel();
		TableColumn column = cmodel.getColumn(0);
		column.setMaxWidth(4);
		column = cmodel.getColumn(1);
		column.setPreferredWidth(150);
		column = cmodel.getColumn(2);
		column.setPreferredWidth(5);
		column.setMinWidth(0);
		//column.setMaxWidth(30);
		column.setCellRenderer(coloured);
		
		DefaultTableCellRenderer righter = new DefaultTableCellRenderer();
		righter.setHorizontalAlignment(SwingConstants.RIGHT);
		pathTable.getColumnModel().getColumn(1).setCellRenderer(righter);
		
		// Create an adjuster to set the path for a key on right click
		JPopupMenu popup = new JPopupMenu ();
		adjuster = new TreeAdjuster(explorer, popup, paths, lister);
		explorer.addMouseListener(adjuster);
		
		// Create a refresher
		refresher = new Refresher();
		refresher.add(paths);
		refresher.add(lister);
		refresher.add(explorer);

		// Create dialogue for colours menu        
		colours = new ColourDialogue(10, coloured, refresher);
		makeMenus();

		// Assign function and other key actions
		ArrayList actionables = new ArrayList();
		actionables.add(explorer);
		actionables.add(mainPanel);
		actionables.add(lister);
		actionables.add(pathTable);

		// ^Z = undo move
		UndoMoveAction undo = new UndoMoveAction(10, lister, refresher);
		assignKeyAction(KeyEvent.VK_Z, KeyEvent.CTRL_MASK, "undo", undo, actionables);
        
		// Escape = refresh
		RefreshAction refresh = new RefreshAction(refresher);
		assignKeyAction(KeyEvent.VK_ESCAPE, 0, "escape", refresh, actionables);
		
		// Delete = delete
		deleter = new DeleteAction(lister, refresher);
		assignKeyAction(KeyEvent.VK_DELETE, 0, "delete", deleter, actionables);
		
		// Insert = make directory
		MkDirAction mkdir = new MkDirAction(explorer);
		assignKeyAction(KeyEvent.VK_INSERT, 0, "insert", mkdir, actionables);
		
		// Enter = rename file
		RenameAction rename = new RenameAction(lister, refresher);
		assignKeyAction(KeyEvent.VK_ENTER, 0, "rename", rename, actionables);
        
        // Current path set number
        SpinnerNumberModel pathSets = new SpinnerNumberModel(paths.getPathSet(), 0, nSets-1, 1);
        JSpinner pathSpinner = new JSpinner(pathSets);
        pathSpinner.setMaximumSize(new Dimension(200, 20));
        pathSpinner.setMinimumSize(new Dimension(200, 20));
        pathSpinner.addChangeListener(new SpinnerListener(pathSets, paths));

        // Assemble the components
        JPanel list = new JPanel();
        list.setLayout(new BoxLayout(list, BoxLayout.Y_AXIS));
        list.add(listPanel);
        Box fpBox = new Box(BoxLayout.X_AXIS);
        fpBox.add(Box.createHorizontalGlue());
        fpBox.add(fileCounter);
        fpBox.add(Box.createHorizontalGlue());
        fpBox.add(pathSpinner);
        fpBox.add(Box.createHorizontalGlue());
        list.add(fpBox);
        list.add(pathTable.getTableHeader());
        list.add(pathTable);
        list.doLayout();

		pane1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, treePanel, list);        
		pane2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pane1, image);
		mainPanel.add(pane2);
		
		actionables.add(pane1);
		actionables.add(pane2);
		
		// Assign function keys
		for (int i = 0 ; i < nKeys ; ++i)
		{
			String key = Integer.toString(i+1);
			MoveAction action = new MoveAction(i, paths, lister, undo, refresher);

			assignKeyAction(codes[i], 0, key, action, actionables);
			assignKeyAction(digit[i], 0, key, action, actionables);
            
			JMenuItem menuitem = new JMenuItem("F"+Integer.toString(i+1));
			popup.add(menuitem);
			menuitem.addActionListener(new AssignAction(i, paths, adjuster, lister));

            for (int j = 0; j < nSets; ++j)
            {
    			paths.setPath(i, j, null);
            }
		}
        
        // Add encryption entries to popup menu
        popup.addSeparator();
        JMenuItem menuitem = new JMenuItem("Encrypt");
        popup.add(menuitem);
        menuitem.setName("EncryptDir");
        menuitem.addActionListener(this);
        menuitem = new JMenuItem("Encrypt recursively");
        popup.add(menuitem);
        menuitem.setName("EncryptDirRec");
        menuitem.addActionListener(this);
        menuitem = new JMenuItem("Decrypt");
        popup.add(menuitem);
        menuitem.setName("DecryptDir");
        menuitem.addActionListener(this);
        menuitem = new JMenuItem("Decrypt recursively");
        popup.add(menuitem);
        menuitem.setName("DecryptDirRec");
        menuitem.addActionListener(this);
        
        passwordManager = new PasswordManager();

        // Get properties
        if (args.length > 0)
        {
            propLoc = new File(args[0]);
        }
        else
        {
    		// Apply properties
            String propdir = System.getProperty("user.home", "");
            propLoc = new File(propdir, "feast.properties");
        }
        
		props = new PropertyHolder();
		props.add(this);
		props.add(extensions);
		props.add(coloured);
		props.add(paths);
		props.add(lister);
		props.add(explorer);
        props.add(passwordManager);
		props.load(propLoc);
        
        // Handle passwords and encryption
        crypter = null;
        if (passwordManager.hasPassword())
        {
            String password = PasswordDialogue.getPassword("Enter password", null);
            try
            {
                crypter = new Crypter(cryptType, password);
                if (!passwordManager.isOK(crypter, password))
                {
                    JOptionPane.showMessageDialog(null, "Wrong password", "Wrong password",
                    JOptionPane.ERROR_MESSAGE);
                    System.exit(-1);
                }
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(null, "Encryption error", "Encryption error",
                    JOptionPane.ERROR_MESSAGE);
                System.exit(-1);
            }
        }
        
        // Set the crypter for the images. Once this is done, we can enable display, and
        // force a refresh
        imagePanel.setCrypter(crypter);
        imagePanel.setEnabled(true);
        lister.refresh(0);
		
		setMenuState();
		setVisible(true);
		setTitle("FEAST");
		lister.requestFocus();
		
		//explorer.addMouseListener(this);
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent we)
			{
				props.save(propLoc, "FEAST properties");
				System.exit(0);
			}
		});
	}

	/** Create the menus */
	private void makeMenus()
	{
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		JMenu menu = new JMenu("File");
		JMenuItem mitem = new JMenuItem("Load settings");
		mitem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK));
		mitem.addActionListener(this);
		mitem.setName("MenuLoad");
		menu.add(mitem);

		mitem = new JMenuItem("Save settings");
		mitem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_MASK));
		mitem.addActionListener(this);
		mitem.setName("MenuSave");
		menu.add(mitem);

		mitem = new JMenuItem("Save settings as");
		mitem.addActionListener(this);
		mitem.setName("MenuSaveAs");
		menu.add(mitem);

        menu.addSeparator();

        mitem = new JMenuItem("Set password");
        mitem.addActionListener(this);
        mitem.setName("MenuSetPassword");
        menu.add(mitem);

		menu.addSeparator();
		mitem = new JMenuItem("Exit");
		mitem.addActionListener(this);
		mitem.setName("MenuExit");
		menu.add(mitem);

		menuBar.add(menu);

		menu = new JMenu("Settings");
		
		mitem = new JMenuItem("Random");
		mitem.addActionListener(this);
		mitem.setName("MenuRandom");
		menu.add(mitem);

		mitem = new JRadioButtonMenuItem("Confirm deletes");
		mitem.addActionListener(this);
		mitem.setName("MenuConfirm");
		menu.add(mitem);
		deleterMenu = mitem;

		mitem = new JMenuItem("Colours");
		mitem.addActionListener(this);
		mitem.setName("MenuColours");
		menu.add(mitem);

		mitem = new JRadioButtonMenuItem("Pin file list");
		mitem.addActionListener(this);
		mitem.setName("MenuPin");
		mitem.setSelected(lister.isPinned());
		menu.add(mitem);
		pinMenu = mitem;
		
		mitem = new JRadioButtonMenuItem("List full paths");
		mitem.addActionListener(this);
		mitem.setName("MenuFullPath");
		mitem.setSelected(paths.isFullPaths());
		menu.add(mitem);
		fullPathsMenu = mitem;
		
        mitem = new JRadioButtonMenuItem("Mangle names");
        mitem.addActionListener(this);
        mitem.setName("MenuMangle");
        mitem.setSelected(mangleNames);
        menu.add(mitem);
        mangleMenu = mitem;
        
		JMenu sort = new JMenu("File sort order");
		menu.add(sort);

		mitem = new JRadioButtonMenuItem("Name, ascending");
		mitem.addActionListener(this);
		mitem.setName("MenuSortNameAsc");
		sort.add(mitem);
		sortNAMenu = mitem;

		mitem = new JRadioButtonMenuItem("Name, descending");
		mitem.addActionListener(this);
		mitem.setName("MenuSortNameDes");
		sort.add(mitem);
		sortNDMenu = mitem;
		
		mitem = new JRadioButtonMenuItem("Size, ascending");
		mitem.addActionListener(this);
		mitem.setName("MenuSortSizeAsc");
		sort.add(mitem);
		sortSAMenu = mitem;
		
		mitem = new JRadioButtonMenuItem("Size, descending");
		mitem.addActionListener(this);
		mitem.setName("MenuSortSizeDes");
		sort.add(mitem);
		sortSDMenu = mitem;
		
        mitem = new JRadioButtonMenuItem("Random");
        mitem.addActionListener(this);
        mitem.setName("MenuSortRandom");
        sort.add(mitem);
        sortSRMenu = mitem;
        
		menuBar.add(menu);
		
		menu = new JMenu("Help");
		mitem = new JMenuItem("About");
		mitem.addActionListener(this);
		mitem.setName("MenuAbout");
		menu.add(mitem);

		menuBar.add(menu);
	}
	
	/** Set state information in menu */
	private void setMenuState()
	{
		pinMenu.setSelected(lister.isPinned());
        fullPathsMenu.setSelected(paths.isFullPaths());
        mangleMenu.setSelected(mangleNames);
		
		int order = lister.getSortOrder();
		sortNAMenu.setSelected(order == Lister.SORT_NAME_ASCENDING);
		sortNDMenu.setSelected(order == Lister.SORT_NAME_DESCENDING);
		sortSAMenu.setSelected(order == Lister.SORT_SIZE_ASCENDING);
        sortSDMenu.setSelected(order == Lister.SORT_SIZE_DESCENDING);
        sortSRMenu.setSelected(order == Lister.SORT_RANDOM);
	}
	
	/** Load properties.
	 * @param p Properties object.
	 */
	public void loadProperties(Properties p)
	{
		int xSize = Integer.parseInt(p.getProperty("MainXSize", "800"));
		int ySize = Integer.parseInt(p.getProperty("MainYSize", "600"));
		int xLocn = Integer.parseInt(p.getProperty("MainXLocn", "100"));
		int yLocn = Integer.parseInt(p.getProperty("MainYLocn", "100"));
		setSize(xSize, ySize);
		setLocation(xLocn, yLocn);

		int loc = Integer.parseInt(p.getProperty("MainDiv1", "100"));
		pane1.setDividerLocation(loc);
		loc = Integer.parseInt(p.getProperty("MainDiv2", "225"));
		pane2.setDividerLocation(loc);

		String s = p.getProperty("MainDeleteConfirm", "true");
		deleter.setConfirm(Boolean.valueOf(s).booleanValue());
		deleterMenu.setSelected(deleter.getConfirm());

        s = p.getProperty("MainMangleNames", "false");
        mangleNames = Boolean.valueOf(s).booleanValue();
        mangleMenu.setSelected(mangleNames);
	}
	
	/** Save properties to external file.
	 * @param p Properties object.
	 */
	public void saveProperties(Properties p)
	{
		Dimension d = getSize();
		p.setProperty("MainXSize", Integer.toString((int) d.getWidth()));
		p.setProperty("MainYSize", Integer.toString((int) d.getHeight()));
		
		Point l = getLocation();
		p.setProperty("MainXLocn", Integer.toString((int) l.getX()));
		p.setProperty("MainYLocn", Integer.toString((int) l.getY()));

		int loc = pane1.getDividerLocation();
		p.setProperty("MainDiv1", Integer.toString(loc));
		loc = pane2.getDividerLocation();
		p.setProperty("MainDiv2", Integer.toString(loc));
		
		p.setProperty("MainDeleteConfirm", Boolean.toString(deleter.getConfirm()));
        p.setProperty("MainMangleNames", Boolean.toString(mangleNames));
	}		
	
	/** Mouse handler.
	 * @param me Mouse event
	 */
	public void mouseEntered(MouseEvent me) {}
	
	/** Mouse handler.
	 * @param me Mouse event
	 */
	public void mouseExited(MouseEvent me) {}
	
	/** Mouse handler.
	 * @param me Mouse event
	 */
	public void mousePressed(MouseEvent me) {}
	
	/** Mouse handler.
	 * @param me Mouse event
	 */
	public void mouseReleased(MouseEvent me) {}
	
	/** Mouse handler.
	 * @param me Mouse event
	 */
	public void mouseClicked(MouseEvent me)	{}
	
	/** Utility function for assigning a key action
	 * 
	 * @param code Key code
	 * @param mod Key modifiers
	 * @param key Key name
	 * @param action The action for this key
	 * @param dest The items to assign the action to
	 */
	private void assignKeyAction(int code, int mod, String key, AbstractAction action, Collection dest)
	{
		for (Iterator i = dest.iterator(); i.hasNext(); )
		{
			JComponent d = (JComponent) i.next();
			d.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
					.put(KeyStroke.getKeyStroke(code, mod), key);
			d.getActionMap().put(key, action);
			d.registerKeyboardAction(action, KeyStroke.getKeyStroke(code, mod), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		}
	}
	
    /** Action handler for menu.
     * @param e The action event.
     */
    public void actionPerformed(ActionEvent e)
    {
		JMenuItem source = (JMenuItem)(e.getSource());
		String name = source.getName();
		
		if (name.equals("MenuLoad"))
		{
			chooser.setDialogTitle("Load settings");
			int rc = chooser.showOpenDialog(this);
			if (rc == JFileChooser.APPROVE_OPTION)
			{
				props.load(chooser.getSelectedFile());
				propLoc = chooser.getSelectedFile();
			}
		}
		else if (name.equals("MenuSave"))
		{
			props.save(propLoc, "FEAST settings");
		}
		else if (name.equals("MenuSaveAs"))
		{
			chooser.setDialogTitle("Save settings as");
			int rc = chooser.showSaveDialog(this);
			if (rc == JFileChooser.APPROVE_OPTION)
			{
				props.save(chooser.getSelectedFile(), "FEAST settings");
				propLoc = chooser.getSelectedFile();
			}
		}
        else if (name.equals("MenuSetPassword"))
        {
            String password = PasswordDialogue.getPassword("New password", this);
            if (password != null)
            {
                String confirm  = PasswordDialogue.getPassword("Confirm password", this);
                if (confirm != null && confirm.equals(password))
                {
                    try
                    {
                        crypter = new Crypter(cryptType, password);
                        passwordManager.setPassword(crypter, password);
                        imagePanel.setCrypter(crypter);
                    }
                    catch (Exception exc)
                    {
                        JOptionPane.showMessageDialog(null, "Encryption error",
                            "Password not changed",
                            JOptionPane.ERROR_MESSAGE);
                    }
                }
                else
                {
                    JOptionPane.showMessageDialog(null, "Not confirmed",
                        "Password not changed",
                        JOptionPane.ERROR_MESSAGE);
                }
            }
        }
		else if (name.equals("MenuExit"))
		{
			processEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
		}
		else if (name.equals("MenuRandom"))
		{
			lister.getRandomFromUser();
		}
		else if (name.equals("MenuConfirm"))
		{
			deleter.setConfirm(source.isSelected());
		}
		else if (name.equals("MenuColours"))
		{
			colours.show(this);
		}
		else if (name.equals("MenuPin"))
		{
			lister.setPinned(source.isSelected());
			refresh(0);
		}
		else if (name.equals("MenuFullPath"))
		{
			paths.setFullPaths(source.isSelected());
		}
        else if (name.equals("MenuMangle"))
        {
            mangleNames = source.isSelected();
        }
		else if (name.equals("MenuSortNameAsc"))
		{
			lister.setSortOrder(Lister.SORT_NAME_ASCENDING);
			refresh(0);
		}
		else if (name.equals("MenuSortNameDes"))
		{
			lister.setSortOrder(Lister.SORT_NAME_DESCENDING);
			refresh(0);
		}
		else if (name.equals("MenuSortSizeAsc"))
		{
			lister.setSortOrder(Lister.SORT_SIZE_ASCENDING);
			refresh(0);
		}
        else if (name.equals("MenuSortSizeDes"))
        {
            lister.setSortOrder(Lister.SORT_SIZE_DESCENDING);
            refresh(0);
        }
        else if (name.equals("MenuSortRandom"))
        {
            lister.setSortOrder(Lister.SORT_RANDOM);
            refresh(0);
        }
		else if (name.equals("MenuAbout"))
		{
			JOptionPane.showMessageDialog(this, "FEAST by MooseCrowSoft Inc.\nwww.friendlymoose.com",
			"About FEAST", JOptionPane.INFORMATION_MESSAGE);
		}
        else if (name.equals("EncryptDir"))
        {
            cryptdir(false, false);
        }
        else if (name.equals("EncryptDirRec"))
        {
            cryptdir(false, true);
        }
        else if (name.equals("DecryptDir"))
        {
            cryptdir(true, false);
        }
        else if (name.equals("DecryptDirRec"))
        {
            cryptdir(true, true);
        }
    }
    
    /** Execute a refresh. Invokes each refreshable object and then sets the menu.
     * 
     * @param level Refresh level.
     */
    public void refresh(int level)
    {
    	refresher.refresh(level);
    	setMenuState();
    }
    
    /** En/Decrypt directory given by tree adjuster.
     *  @param pDecrypt True for decrypt, false for encrypt.
     *  @param pRecurse True to recurse.
     */
    private void cryptdir(boolean pDecrypt, boolean pRecurse)
    {
        if (!passwordManager.hasPassword())
        {
            JOptionPane.showMessageDialog(null, "Can't do it", "You must set a password first",
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        File dir = adjuster.getFile();
        if (dir == null || !dir.isDirectory())
        {
            return;
        }

        // Make file list
        List files = new LinkedList();
        getAllFiles(files, dir, pRecurse);
        
        for (Iterator i = files.iterator(); i.hasNext(); )
        {
            File file = (File) i.next();
            String name = file.getAbsolutePath();
            try
            {
                if (pDecrypt)
                {
                    // Mangling disabled!
                    crypter.decryptFile(name, EncryptExt, false && mangleNames);
                }
                else
                {
                    // Mangling disabled!
                    crypter.encryptFile(name, EncryptExt, false && mangleNames);
                }
            }
            catch (Exception e)
            {
                System.err.println("Cryption failed on "+name);
            }
        }
        
        lister.refresh(0);
    }
    
    /** Get all files in directory, recursing if asked.
     *  @param pFiles File list.
     *  @param pDir Directory.
     *  @param pRecurse True to recurse.
     */
    private void getAllFiles(List pFiles, File pDir, boolean pRecurse)
    {
        File files[] = pDir.listFiles();
        for (int i = 0; i < files.length; ++i)
        {
            File file = files[i];
            if (file.isDirectory())
            {
                if (pRecurse)
                {
                    getAllFiles(pFiles, file, pRecurse);
                }
            }
            else if (file.isFile())
            {
               pFiles.add(file);
            }
        }
    }
    
	/** Entry point.
	 * 
	 * @param args Command line arguments.
	 */
	public static void main(String[] args)
	{
		new Feast(args);
	}

}
