//A component of FEAST.
//Copyright (c) 2003 David Elworthy.
//For information, visit www.friendlymoose.com
package com.friendlymoose.feast;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.swing.JPanel;

/** Panel for displaying images.
 * A click in the panel toggles between scaled and full-size.
 * However, we only ever scale down, not up.
 * Overlays the name onto the image.
 */
public class ImagePanel extends JPanel
{
	/** The image itself. */
    private Image image;
    
    /** Name of image file */
    private String filename;
    
    /** Size of image file */
    private long filesize;
    
    /** Width and height of image */
    private int width, height;
    
    /** True if scaled to fit */
    private boolean scaled;
    
    /** Colour for text showing image details. */
    private Color textColour = null;
    
    /** A media tracker */
	private MediaTracker tracker;
    
    /** Crypter for encrypted files. */
    private Crypter crypter = null;
    
    /** Enabled state. If false, the display does not happen. */
    private boolean enabled = false;

	/** Create image panel.
     *  @param pEnabled Set initial enabled state.
	 */
    public ImagePanel(boolean pEnabled)
    {
        super();
        scaled = true;
        setTextColour(Color.CYAN);
        enabled = pEnabled;
        displayFile(null);

        addMouseListener(new MouseListener()
        {
            public void mouseClicked(MouseEvent e)
            {
                ((ImagePanel) e.getComponent()).invertScaled();

            }
            public void mouseEntered(MouseEvent e) {}
            public void mouseExited(MouseEvent e) {}
            public void mousePressed(MouseEvent e) {}
            public void mouseReleased(MouseEvent e) {}
        });
        
        tracker = new MediaTracker(this);
    }
    
    /** Set enabled state. Does NOT cause a refresh.
     *  @param pEnabled Enabled state.
     */
    public void setEnabled(boolean pEnabled)
    {
        enabled = pEnabled;
    }
    
    /** Set the crypter.
     * @param pCrypter The crypter.
     */
    public void setCrypter(Crypter pCrypter)
    {
        crypter = pCrypter;
    }
    
	/** Set text colour. Applies on next change.
	 * @param colour The text colour
	 */
	public void setTextColour(Color colour)
	{
		textColour = colour;
	}

	/** Change scaled setting.
	 * 
	 * @param setting True for scaled display
	 */
	public void setScaled(boolean setting)
	{
		if (scaled != setting)
		{
			scaled = setting;
			scrollCheck();
			repaint();
		}
	}

	/** Toggle scaled setting.
	 */
    public void invertScaled()
    {
        setScaled(!scaled);
    }

	/** Place scroll bars if needed.
	 */
    public void scrollCheck()
    {
        if (!scaled)
        {
            setPreferredSize(new Dimension(width, height));
        }
        else
        {
            Rectangle vis = getVisibleRect();
            int w = vis.width;
            int h = vis.height;
            setPreferredSize(new Dimension(w, h));
        }
        scrollRectToVisible(new Rectangle(0, 0, 1, 1));
        revalidate();            
    }
    
	/** Set the file to display.
	 * If we are not enabled, we do nothing. The existing image is not removed.
	 * @param newfile The file to display
	 */
    public void displayFile(String newfile)
    {
        if (!enabled)
        {
            return;
        }
        
    	if (image != null)
    	{
    		tracker.removeImage(image);
    	}
        
        if (newfile == null)
        {
            image    = null;
            filename = null;
            filesize = 0;
        }
        else if (newfile.equals(filename))
        {
            return;
        }
        else
        {
            try
            {
                byte imageBytes[];
            
                File imageFile = new File(newfile);
                filesize = imageFile.length();

                InputStream stream = new BufferedInputStream(new FileInputStream(imageFile));

                // Read the image and then pass to the tracker.
                if (crypter != null && newfile.endsWith(Feast.EncryptExt))
                {
                    // Decode into a byte stream, then grab the bytes
                    ByteArrayOutputStream decoded = new ByteArrayOutputStream((int) filesize);
                    crypter.decode(stream, decoded, true);
                    decoded.flush();
                    imageBytes = decoded.toByteArray();
                }
                else
                {
                    imageBytes = new byte[(int) filesize];
                    stream.read(imageBytes);
                }
                stream.close();
                image = Toolkit.getDefaultToolkit().createImage(imageBytes);
                
                tracker.addImage(image, 0);
                tracker.waitForID(0);
            }
            catch (Exception e)
            {
				System.out.println("Image "+newfile+": "+e.toString());
            }
            
            filename = newfile;
            width = image.getWidth(this);
            height = image.getHeight(this);
            scrollCheck();
        }

        repaint();
    }

	/** Display the image, scaling if needed.
	 * @param g Graphics object
	 * 
	 */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if (image == null)
        {
            return;
        }
        try
        {
            int h = height;
            int w = width;
            int panelWidth  = getWidth();
            int panelHeight = getHeight();

            if (scaled)
            {                
                if (width <= panelWidth)
                {
                	if (height > panelHeight)
                	{
                    	w = (int)(width * ((double) panelHeight) / height);
                    	h = panelHeight;
                	}
                }
                else if (height <= panelHeight)
                {
                    h = (int)(height * ((double) panelWidth) / width);
                    w = panelWidth;
                }
                else
                {
                    double scaleH = (double) panelHeight / height;
                    double scaleW = (double) panelWidth  / width;
                    double scale =  (scaleH < scaleW) ? scaleH : scaleW;
                    w = (int)(width  * scale);
                    h = (int)(height * scale);
                }
            }

            int x = (panelWidth  - w)/2;
            int y = (panelHeight - h)/2;

            g.drawImage(image, x, y, w, h, this);
            if (textColour != null)
            {
	            g.setColor(textColour);
	            g.drawString(filename+" - "+width+"x"+height+" - "+filesize, 5, panelHeight-10);
	        }
        }
        catch (NullPointerException npe)
        {
        }
    }
}    
