//A component of FEAST.
//Copyright (c) 2003 David Elworthy.
//For information, visit www.friendlymoose.com
package com.friendlymoose.feast;
import java.awt.Component;
import java.awt.Cursor;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Properties;
import java.util.Random;

import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/** List of files with certain extensions in a given directory.
 * Can call getCount() to find out how many there are.
 */
public class Lister extends JList implements Refreshable, PropertyBearing
{
	/** List of interesting extensions */
    private Extensions extensions;
    
    /** Current directory being displayed */
    private File currentDir = null;
    
    /** Last directory given to refresh. */
    private String lastDir = null;
    
	/** Counter: if non-null displays file count */
    private JLabel counter = null;
    
    /** Random number */
    private Random random;
    
    /** Level: do random jump after this many operations */
    private int randLevel;
    
    /** Count until random jump */
    private int randCount;
    
    /** True if pinned, i.e. doesn't change when selected dir does */
    private boolean pinned = false;
    
    /** Current sort order */
    private int sortOrder = SORT_NAME_ASCENDING;
    
    /** Lengths cache: used to try to improve performance when sorting by length. */
    private Map lengths = new HashMap();
    
	/** Constant to sort by ascending file name */
	public static final int SORT_NAME_ASCENDING  = 0;
	
	/** Constant to sort by descending file name */
	public static final int SORT_NAME_DESCENDING = 1;
	
	/** Constant to sort by ascending file size */
	public static final int SORT_SIZE_ASCENDING  = 2;
	
	/** Constant to sort by descending file size */
	public static final int SORT_SIZE_DESCENDING = 3;
	
    /** Constant to sort randomly */
    public static final int SORT_RANDOM = 4;
    
	/** Constant for no selected item */
	public static final int NOINDEX = -99999;

	/** Renderer for list cells. */
	private static class CellRenderer extends JLabel implements ListCellRenderer
	{
		/** Get the cell renderer.
		 * @param list (list)
		 * @param value (value)
		 * @param index (index)
		 * @param isSelected (isSelected)
		 * @param cellHasFocus (cellHasFocus)
		 * @return Renderer component
		 */
		 public Component getListCellRendererComponent
		 (JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		 {
		 	File f = (File) value;
			String s = f.getName();
			setText(s);
		    if (isSelected)
		    {
		    	setBackground(list.getSelectionBackground());
		    	setForeground(list.getSelectionForeground());
		    }
			else
			{
			   setBackground(list.getBackground());
			   setForeground(list.getForeground());
		    }
		    setEnabled(list.isEnabled());
		    setFont(list.getFont());
			setOpaque(true);
			return this;
		 }
	 }
	 
	 /** Inner class for comparing files for sorted display */
	 public static class FileSorter implements Comparator
	 {
	 	/** Current sort order */
	 	private int sortOrder;
        
        /** Lengths map */
        private Map lengthMap;
	 	
	 	/** Construct a file sorter
	 	 * 
	 	 * @param order Order code for sorting.
         * @param lengths Mapping from name to length (may be null if not sorting on length).
	 	 */
	 	public FileSorter(int order, Map lengths)
	 	{
	 		sortOrder = order;
            lengthMap = lengths;
	 	}

		/** Sort files.
		 * @param arg0 A File
		 * @param arg1 Another file
		 * @return The ordering for the files.
		 */
        public int compare(Object arg0, Object arg1)
        {
        	if (arg0 == arg1)
        	{
        		return 0;
        	}
        		
			File f0 = (File) arg0;
			File f1 = (File) arg1;
			if (sortOrder == SORT_NAME_ASCENDING)
			{
				return f0.getName().compareTo(f1.getName());
			}
			else if (sortOrder == SORT_NAME_DESCENDING)
			{
				return f1.getName().compareTo(f0.getName());
			}
            else
            {
                Integer i = (Integer) lengthMap.get(f0.getName());
                int len0 = (i != null) ? i.intValue() : (int) f0.length();
                i = (Integer) lengthMap.get(f1.getName());
                int len1 = (i != null) ? i.intValue() : (int) f1.length();
                int diff = len0 - len1;
                
                if (sortOrder == SORT_SIZE_ASCENDING)
    			{
    				return (diff != 0) ? diff : f0.getName().compareTo(f1.getName());
    			}
    			else if (sortOrder == SORT_SIZE_DESCENDING)
    			{
    				return (diff != 0) ? -diff : f0.getName().compareTo(f1.getName());
    			}
            }
            return 0;
            
        }
	 }
	 
	 /** Inner class for updating count */
	 private class SelectionListener implements ListSelectionListener
	 {
	 	/** Update count.
	 	 * @param arg0 Action event
	 	 */
        public void valueChanged(ListSelectionEvent arg0)
        {
        	showCount();
        }
	 }
    
    /** Create the lister over given extensions.
     * 
     * @param ext Extension selector.
     * @param count If non-null, a label to display the total file count.
     */
    public Lister(Extensions ext, JLabel count)
    {
        super();

        extensions = ext;
        counter = count;
        random = new Random();
        randLevel = 0;
        setCellRenderer(new CellRenderer());
        addListSelectionListener(new SelectionListener());
    }
    
    /** Set random level. A jump to a random location in the list happens after
     * "level" operations.
     * @param level Random jump level
     */
    public void setRandom(int level)
    {
    	randLevel = level;
    	randCount = level;
    }
    
    /** Force a random jump on next refresh. */
    public void forceRandom()
    {
    	randCount = 0;
    }
    
    /** Set the new sort order. Needs a refresh to take effect.
     * 
     * @param order The sort order.
     */
	public void setSortOrder(int order)
	{
		sortOrder = order;
	}
    
    /** Get code for current sort order.
     *  
     * @return Current sort order.
     */
	public int getSortOrder()
	{
		return sortOrder;
	}
    
    /** Set the list to a given directory
     * 
     * @param dir Directory. If null or not a directory, clear the list.
     */
    public void setList(File dir)
    {
		currentDir = dir;
		refresh(0);
    }
    
    /** Get the name at given position. May be null.
     * 
     * @param i Position in the list
     * @return Name at this position.
     */
    public String getName(int i)
    {
        if (i < 0)
        {
            return null;
        }

        ListModel model = getModel();
        if (i >= model.getSize())
        {
            return null;
        }

        File file = (File) model.getElementAt(i);
        try
        {
            return (file == null) ? null : file.getCanonicalPath();
        }
        catch (IOException e)
        {
            return null;
        }
    }
    
	/** Get currently selected file. May be null.
	 * 
	 * @return currently selected file, if any.
	 */
	public File getSelected()
	{
		return (File) getSelectedValue();
	}

	/** Set currently selected to file or index.
	 *  @param selFile File to select. If null use index.
	 *  @param index Index to select if file is null or not found.
	 */
	public void setSelected(File selFile, int index)
	{
		if (selFile != null)
		{
			setSelectedValue(selFile, true);
			if (getSelectedValue() != null)
			{
				return;
			}
		}

		if (index == NOINDEX)
		{
			return;
		}
			
		if (index < 0)
		{
			index = 0;
		}
			
		int max = getModel().getSize();
		if (index >= max)
		{
			index = max-1;
		}
		
		if (index >= 0)
		{
			setSelectedIndex(index);
			ensureIndexIsVisible(index);
		}
	}
	
	/** Randomly jump to a location in the list. */
	public void randomJump()
	{
		int length = getModel().getSize();
		if (length > 0)
		{
			setSelected(null, random.nextInt(length));
			randCount = randLevel;
		}
	}

	/** Refresh from the current directory.
	 *  We try to restore the old selection if possible.
	 *  @param level Refresh level. If >=1, we do a random jump if randomness is
	 *  set.
	 */
	public void refresh(int level)
	{
		Cursor hourglassCursor = new Cursor(Cursor.WAIT_CURSOR);
		setCursor(hourglassCursor);
 
 		if (level >= 1)
 		{
			forceRandom();
 		}

		if (currentDir != null && currentDir.isDirectory())
		{
			File selFile  = getSelected();
			int  selIndex = getSelectedIndex();
			boolean found = false;
            
            String dirName = "";
            try
            {
                dirName = currentDir.getCanonicalPath();
            }
            catch (IOException e)
            {
            }
            
            if (lastDir == null || dirName.equals(lastDir))
            {
                lastDir = dirName;
                lengths.clear();
            }
			
			File[] list = currentDir.listFiles(extensions);
			
			if (list != null)
			{
				LinkedList files = new LinkedList();
				for (int i = 0 ; i < list.length ; i++)
				{
                    File f = list[i];
					if (f.isFile ())
					{
						files.add(f);
                        String name = f.getName();
                        if (lengths.get(name) == null)
                        {
                            lengths.put(name, new Integer((int) f.length()));
                        }
                        
						if (!found && selFile != null && selFile.equals(f))
						{
							found = true;
						}
					}
				}
	            
                if (sortOrder == SORT_RANDOM)
                {
                    Collections.shuffle(files);
                    list = (File[]) files.toArray(new File[0]);
                }
                else
                {
                    list = (File[]) files.toArray(new File[0]);
	                Arrays.sort(list, new FileSorter(sortOrder, lengths));
                }
				setListData(list);
	            
				// Set selected index
				if (randLevel > 0 && --randCount <= 0)
				{
					randomJump();
				}
				else
				{
					setSelected((found) ? selFile : null, selIndex);
					if (getSelectedIndex() != -1)
					{
						requestFocus();
					}
				}
			}
						
			showCount();
		}
		else
		{
			setListData(new Object[0]);
			if (counter != null)
			{
				counter.setText("");
			}
		}
		
		ensureIndexIsVisible(getSelectedIndex());
		Cursor normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);
		setCursor(normalCursor);
    }
    
    /** Get count of items.
     * @return Count of items (i.e. files).
     */
    public int getCount()
    {
    	return getModel().getSize();
    }

	/** Display the count on the selected directory. */
	public void showCount()
	{
		if (counter != null)
		{
			counter.setText((1+getSelectedIndex())+"/"+getCount());
		}
	}

	/** Load properties.
	 * @param props Properties object
	 */
    public void loadProperties(Properties props)
    {
		String s;
		s = props.getProperty("ListerSort", "0");
		sortOrder = Integer.parseInt(s);

    	s = props.getProperty("ListerDir", null);
    	if (s != null)
    	{
    		currentDir = new File(s);
    		refresh(0);
    	}

		s = props.getProperty("ListerFile", null);
		
		File selFile = (s == null) ? null : new File(s);
		if (selFile != null && (!selFile.exists() || !selFile.isFile()))
		{
			selFile = null;
		}

		s = props.getProperty("ListerIndex", "0");
		int selIndex = Integer.parseInt(s);
		setSelected(selFile, selIndex);
		
		s = props.getProperty("ListerRand", "-1");
		randLevel = Integer.parseInt(s);

		s = props.getProperty("ListerPin", "false");
		pinned = Boolean.valueOf(s).booleanValue();
    }

    /** Save properties.
     * @param props Properties object
     */
    public void saveProperties(Properties props)
    {
    	if (currentDir != null)
    	{
    		props.setProperty("ListerDir", currentDir.toString());
    	}
    	
		File selFile  = getSelected();
		if (selFile != null)
		{
			props.setProperty("ListerFile", selFile.toString());
		}
			
		int selIndex = getSelectedIndex();
		props.setProperty("ListerIndex", Integer.toString(selIndex));

		props.setProperty("ListerRand", Integer.toString(randLevel));
		props.setProperty("ListerPin",  Boolean.toString(pinned));
		props.setProperty("ListerSort", Integer.toString(sortOrder));

    }

	/** Get random jump setting from user */
	public void getRandomFromUser()
	{
		String inputValue
		 = JOptionPane.showInputDialog("Random jump after (0=disable)",
		 			Integer.toString(randLevel)); 
			 
		if (inputValue != null)
		{
			randLevel = Integer.parseInt(inputValue);
		}
	}

	/** Report if pinned.
	 * 
	 * @return True if list is pinned
	 */
    public boolean isPinned()
    {
        return pinned;
    }

	/** Set pinned status. When pinned, the lister does not change if the
	 * selected directory does.
	 * @param b True to pin the list.
	 */
    public void setPinned(boolean b)
    {
        pinned = b;
    }

}



