//A component of FEAST.
//Copyright (c) 2003 David Elworthy.
//For information, visit www.friendlymoose.com
package com.friendlymoose.feast;
import java.io.File;
import java.util.Properties;

import javax.swing.table.AbstractTableModel;

/** A table of paths.
 * Column 0 is an identifier, 1 is the path, and 2 is the count of entries.
 * Allows multiple path sets
 */
public class PathsTable extends AbstractTableModel
implements Refreshable, PropertyBearing
{
	/** Paths indexed by row and path set */
    private File   paths[][];

	/** Displayed name indexed by row */
    private String texts[];
    
	/** File count indexed by row */
    private int    counts[];
    
    /** Extensions we handle */
    private Extensions extensions;
    
    /** Last row assigned to. */
    private int lastRow = -1;
    
    /** Column names */
    private final String colNames[] = {"F", "Location", "#"};
    
    /** True to display full paths, false for last segment */
    private boolean fullPaths = false;
    
    /** Current path set. */
    private int pathSet = 0;

	/** Create with given number of rows, selecting extensions
	 * 
	 * @param max Number of rows
	 * @param ext Extension selector
     * @param sets Number of path sets
	 */
    public PathsTable(int max, Extensions ext, int sets)
    {
        paths  = new File[max][];
        for (int i = 0; i < max; ++i)
        {
            paths[i] = new File[sets];
        }
        texts  = new String[max];
        counts = new int[max];
        extensions = ext;
        pathSet = 0;
    }
    
	/** Save properties.
	 * @param props Property holder
	 */
	public void saveProperties(Properties props)
	{
		int max  = paths.length;
        int sets = paths[0].length;
        props.setProperty("Paths#", Integer.toString(max));
        props.setProperty("PathSets", Integer.toString(sets));
        props.setProperty("PathsFull",  Boolean.toString(fullPaths));
        props.setProperty("PathLastSet", Integer.toString(pathSet));
		
		for (int i = 0; i < max; ++i)
		{
            for (int j = 0; j < sets; ++j)
            {
    			String s = (paths[i][j] == null) ? "" : paths[i][j].toString();
	   		    props.setProperty("Paths_P"+i+"_"+j, s);
            }
		}
	}
    
	/** Load properties.
	 * @param props Property holder
	 */
	public void loadProperties(Properties props)
	{
		String s = props.getProperty("PathsFull", "false");
		fullPaths = Boolean.valueOf(s).booleanValue();

        int max  = Integer.parseInt(props.getProperty("Paths#", "0"));
        int sets = Integer.parseInt(props.getProperty("PathsSets", "0"));
        pathSet = Integer.parseInt(props.getProperty("PathsLastSet", "0"));
        
		if (max == 0 || max > paths.length)
		{
			max = paths.length;
		}
        if (sets == 0 || sets > paths[0].length)
        {
            sets = paths[0].length;
        }
		
		for (int i = 0; i < max; ++i)
		{
            for (int j = 0; j < sets; ++j)
            {
    			s = props.getProperty("Paths_P"+i+"_"+j, null);
			    setPath(i, j, (s == null || s.equals("")) ? null : new File(s));
            }
		}

		refresh(1);
	}
    
    /** Get column name.
     * @param col Column number.
     * @return Column name.
     */
	public String getColumnName(int col)
	{ 
		return colNames[col];
	}
	
	/** Get number of columns
	 * @return Number of columns
	 */
    public int getColumnCount()
    {
    	return 3;
  	}
    
    /** Get number of rows
     * @return Number of rows
     */
    public int getRowCount()
    {
    	return paths.length;
   	}

	/** Get a cell
	 * @param row Row number
	 * @param col Column number
	 * @return Value in the cell
	 */    
    public Object getValueAt(int row, int col)
    {
        if (col == 0)
    	{
            return Integer.toString(row+1);
    	}
        else if (col == 1)
        {
            return texts[row];
        }
        else if (col == 2)
        {
        	if (paths[row][pathSet] == null)
        	{
        		return "";
        	}
        	else if (paths[row][pathSet].exists())
        	{
	            return new Integer(counts[row]);
        	}
	        else
	        {
	        	return "?";
	        }
        }
        else
        {
        	return null;
        }
    }

	/** Set the path in a given cell.
	 * @param row Row number. If this is -1, we search for the first empty row,
	 * and if we find none, use the one after the last row we assigned.
     * @param set Path set. -1 = current
	 * @param path Path in this row
	 */
    public void setPath(int row, int set, File path)
    {
        if (set == -1)
        {
            set = pathSet;
        }
        
    	if (row == -1)
    	{
    		for (int i = 0; i < paths.length; ++i)
    		{
    			if (paths[i][set] == null)
    			{
    				row = i;
    				break;
    			}
    		}
    	}
    	
    	if (row == -1)
    	{
    		row = lastRow+1;
    		if (row >= paths.length)
    		{
    			row = 0;
    		}
    	}
    	
        paths[row][set] = path;
        
        if (path != null)
        {
			String text = (fullPaths) ? path.toString() : path.getName();
			texts[row] = text;
        }
        else
        {
        	texts[row] = "";
        }
            
        updateCount(row);
        lastRow = row;
        
        // The next call is needed when we update without a real event, 
        // e.g. from a shift/right click
		fireTableCellUpdated(row, 1);
    }

	/** Get the path in a given row.
	 * 
	 * @param row Row number
	 * @return Path in this row
	 */
    public File getPath(int row)
    {
        return paths[row][pathSet];
    }
    
    /** Update the counts in the table.
     * @param level Refresh level (0 = counts, 1 = paths as well)
     *
     */
    public void refresh(int level)
    {
        if (level >= 1)
        {
            for (int i = 0 ; i < paths.length; ++i)
            {
                File path = paths[i][pathSet];
                if (path != null)
                {
                    texts[i] = (fullPaths) ? path.toString() : path.getName();
                }
                else
                {
                    texts[i] = "";
                }
                fireTableCellUpdated(i, 1);
            }
        }
        
        for (int i = 0 ; i < paths.length; ++i)
        {
            updateCount(i);
        }
    }

	/** Update the count in a row
	 * 
	 * @param row Row number
	 */
    public void updateCount(int row)
    {
        if (paths[row] == null)
        {
            counts[row] = 0;
        }
        else
        {
            try
            {
                File dir = paths[row][pathSet];
                File[] list = dir.listFiles(extensions);
                int count = list.length;
                counts[row] = count;
            }
            catch (Exception e)
            {
                counts[row] = 0;
            }
        }

        fireTableCellUpdated(row, 2);
    }

	/** Are we showing full paths?
	 *  
	 * @return True if showing full paths.
	 */
    public boolean isFullPaths()
    {
        return fullPaths;
    }

	/** Set full paths display
	 * 
	 * @param b True to show full paths, false for short version
	 */
    public void setFullPaths(boolean b)
    {
        fullPaths = b;
		for (int i = 0 ; i < paths.length; ++i)
		{
			setPath(i, pathSet, paths[i][pathSet]);
		}
    }
    
    /** Set the current path set and refresh
     *  @param set Set number
     */
    public void setPathSet(int set)
    {
        pathSet = set;
        refresh(1);
    }
    
    /** Get current path set.
     *  @return The path set.
     */
    public int getPathSet()
    {
        return pathSet;
    }

}


